/**
 * @file:
 * Enhances chosen by allowing to sort the selected items.
 */

(function ($, drupalSettings) {

  'use strict';

  Drupal.behaviors.chosen_order = {
    attach: function (context) {

      $(once('chosen-order-init', '.field--widget-chosen-select')).each(function () {
        let fieldName = $(this).attr('data-chosen-order-field-name');
        $(this).find('ul.chosen-choices').sortable({
          filter: 'a',
          delay: 100,
          draggable: '> li.search-choice',
          sort: true,
          update: (event, ui) => {
            Drupal.behaviors.chosen_order.updateFieldOrder(fieldName);
          },
        });

        // Get the entity_ids as saved in db.
        let savedOrder = $('.field-type-chosen-order[data-chosen-order-field-name="' + fieldName + '"]').val().split(',');
        let selectOptions = $(this).find('select option');

        // FIRST - Update chosen order to saved order on page load.
        $(this).find('.chosen-choices li.search-choice').sort(function (a, b) {
          let aIndex = $(a).find('.search-choice-close').attr('data-option-array-index'),
            aValue = selectOptions.eq(aIndex).val(),
            aWeight = Object.keys(savedOrder).find(key => savedOrder[key] === aValue);

          let bIndex = $(b).find('.search-choice-close').attr('data-option-array-index'),
            bValue = selectOptions.eq(bIndex).val(),
            bWeight = Object.keys(savedOrder).find(key => savedOrder[key] === bValue);

          return (bWeight) < (aWeight) ? 1 : -1;
        }).prependTo($(this).find('.chosen-choices'));

        // Update save order if any change event occurs.
        $(this).find('select.chosen-enable').on('change', function(event, params) {
          Drupal.behaviors.chosen_order.updateFieldOrder(fieldName);
        });

        // Update save order on remove choice save order.
        $(this).find('a.search-choice-close').on('click', function(event, params) {
          Drupal.behaviors.chosen_order.updateFieldOrder(fieldName);
        });
      });
    },

    // On update store order of all chosen widgets in hidden fields.
    updateFieldOrder: function (fieldName) {
      let ids = [];

      // Get selected choices.
      $('[data-chosen-order-field-name="' + fieldName + '"] .chosen-choices li a').each(function (i, choice) {
        ids.push($(choice).attr('data-option-array-index'));
      });

      // Map to entity IDs.
      let $options = $('.field--widget-chosen-select[data-chosen-order-field-name="' + fieldName + '"]').find('.chosen-enable').find('option');
      let idsOrder = $.map(ids, function (choice, i) {
        return $($options[choice]).attr('value');
      });

      // Store the order to the field.
      $('.field-type-chosen-order[data-chosen-order-field-name="' + fieldName + '"]').val(idsOrder);
    }
  };

})(jQuery, drupalSettings);
